package org.openxma.dsl.platform.dao;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.openxma.dsl.platform.query.QueryObject;

/**
 * 
 * Defines a generic crud interface for daos.  Additional finders methods are added with interface 
 * inheritance and AOP introductions for concrete implementations. 
 * <p/>
 * Based on the pattern and ideas originated from the following sources:
 * 
 * <ul>
 * 	<li><a href="http://www.ibm.com/developerworks/java/library/j-genericdao.html">Don't repeat the DAO!</a></li>
 * 	<li>Java Persistence with Hibernate 2nd Edition: A generic data-access object pattern</li>
 * 	<li><a href="http://www.adam-bien.com/roller/abien/entry/generic_crud_components_with_java">GENERIC CRUD COMPONENTS WITH JAVA EE 5</a></li>
 * </ul>
 * 
 * @param <T> the parameterized entity type
 * @param <Pk> the identifier corresponding to the given entity type
 */
public interface GenericDao<T, Pk extends Serializable> {
	/**
     * Save or update the given persistent <code>name</code> instance, according to its id (matching the configured "unsaved-value"?).
     *
     * @param entity the instance to save or update.
     */
    void saveOrUpdate(T entity);
    
    /**
     * Copy the state of the given <code>name</code> object onto the persistent <code>name</code> with the same identifier.
     *
     * @param entity the entity instance to merge.
     */
    T merge(T entity);
    
    /**
     * Return the persistent entity instance with the given identifier, throwing an exception if not found.
     * <br><br>
	 * You should not use this method to determine if an instance exists (use <tt>get()</tt>
	 * instead). Use this only to retrieve an instance that you assume exists, where non-existence
	 * would be an actual error.
     *
     * @param oid the identifier of the persistent <code>name</code> instance.
     * @return the persistent instance, or <code>null</code> if not found
     * @throws org.springframework.orm.ObjectRetrievalFailureException if not found.
     */
    T load(Pk oid);
    
    /**
     * Return the persistent instance of the given entity class with the given identifier, or <code>null</code> if not found.
     *
     * @param oid the identifier of the persistent entity instance.
     * @return the persistent instance, or <code>null</code> if not found
     */
    T get(Pk oid);
    
    /**
     * Deletes the persistent entity instance by its primary key.
     *
     * @param oid the identifier of the persistent entity instance to delete.
     */
    void delete(Pk oid);
    
    /**
     * Deletes the persistent instance of the entity <code>getFullyQualifiedName(false)</code>.
     *
     * @param entity the entity instance to delete.
     */
    void delete(T entity);

    /**
     * Execute a query based on the given <code>FilterExpression</code> and <code>OrderSpecification</code>.
     *
     * @param  filterExpression the {@link FilterExpression filterExpression} to to constrain the results to be retrieved.
     * @param  orderSpecification the {@link SortOrderSpecification order} object representing an ordering to be applied to the results.
     * @return a set containing {@link name} instances matching the given filterExpression (never null)
     * @deprecated use {@link #query(QueryObject)} instead
    */
    Set<T> find(FilterExpression filterExpression, SortOrderSpecification orderSpecification);

    /**
     * Return all persistent instances of the given entity class.
     * @return all persistent instances matching the parameterized type.
     * @deprecated use {@link #findAll(SortOrderSpecification)} since 3.6.2
     */
    Set<T> findAll();
    
    /**
     * Return all persistent instances of the given entity class.
     * @param  sortOrderSpecification the {@link SortOrderSpecification order} object representing an ordering to be applied to the results.
     * @return all persistent instances matching the parameterized type.
     */
    Set<T> findAll(SortOrderSpecification sortOrderSpecification);

    /**
	 * Execute a query based on the given example entity object.
	 * @param exampleEntity an instance of the desired entity,
	 * serving as example for "query-by-example"
	 * @return a {@link List} containing 0 or more persistent instances
	 */
    List<T> findByExample(T example);
    
    /**
     * Save changes made to a persistent object.
     * @param transientObject the transientObject to update
     */
    void update(T transientObject);
    
    /**
     * Re-read the state of the given instance from the underlying database.
     * @param object the object to refresh
     */
    void refresh(T object);
    
    /**
     * Remove this instance from the current persistent context.
     * @param object the object to evict
     */
    void evict(T object);
    
    /**
     * Query all instances of the parameterized entity type (E) matching the given queryObject.
     *
     * @param queryObject the criteria expressions to match.
     */
    <E> Collection<E> query(QueryObject queryObject);
    
}
