/*******************************************************************************
 * Copyright (c) 2010 s IT Solutions AT Spardat GmbH . All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/
package org.openxma.dsl.platform.jdbc;

import java.math.BigDecimal;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import org.springframework.jdbc.support.JdbcUtils;
import org.springframework.util.Assert;

/**
 * Initially copied from o.sprfw.j.s.JdbcUtils to additionally support the retrieval of resultSet values by a given
 * column name.
 */
public class JdbcUtils2 extends JdbcUtils {

    /**
     * @param columnName
     *            the SQL name of the column
     * @see #getResultSetValue(ResultSet, int, Class)
     */
    @SuppressWarnings("unchecked")
    public static <T> T getResultSetValue(ResultSet rs, String columName, Class<T> requiredType) throws SQLException {
        Assert.notNull(rs, "parameter 'resultSet' must not be null");
        Assert.notNull(columName, "parameter 'columName' must not be null");
        Assert.notNull(requiredType, "parameter 'requiredType' must not be null");
        Object value = null;
        boolean wasNullCheck = false;

        // Explicitly extract typed value, as far as possible.
        if (String.class.equals(requiredType)) {
            value = rs.getString(columName);
        } else if (boolean.class.equals(requiredType) || Boolean.class.equals(requiredType)) {
            value = Boolean.valueOf(rs.getBoolean(columName));
            wasNullCheck = true;
        } else if (byte.class.equals(requiredType) || Byte.class.equals(requiredType)) {
            value = new Byte(rs.getByte(columName));
            wasNullCheck = true;
        } else if (short.class.equals(requiredType) || Short.class.equals(requiredType)) {
            value = new Short(rs.getShort(columName));
            wasNullCheck = true;
        } else if (int.class.equals(requiredType) || Integer.class.equals(requiredType)) {
            value = new Integer(rs.getInt(columName));
            wasNullCheck = true;
        } else if (long.class.equals(requiredType) || Long.class.equals(requiredType)) {
            value = new Long(rs.getLong(columName));
            wasNullCheck = true;
        } else if (float.class.equals(requiredType) || Float.class.equals(requiredType)) {
            value = new Float(rs.getFloat(columName));
            wasNullCheck = true;
        } else if (double.class.equals(requiredType) || Double.class.equals(requiredType)
                || Number.class.equals(requiredType)) {
            value = new Double(rs.getDouble(columName));
            wasNullCheck = true;
        } else if (byte[].class.equals(requiredType)) {
            value = rs.getBytes(columName);
        } else if (java.sql.Date.class.equals(requiredType)) {
            value = rs.getDate(columName);
        } else if (java.sql.Time.class.equals(requiredType)) {
            value = rs.getTime(columName);
        } else if (java.sql.Timestamp.class.equals(requiredType) || java.util.Date.class.equals(requiredType)) {
            value = rs.getTimestamp(columName);
        } else if (BigDecimal.class.equals(requiredType)) {
            value = rs.getBigDecimal(columName);
        } else if (Blob.class.equals(requiredType)) {
            value = rs.getBlob(columName);
        } else if (Clob.class.equals(requiredType)) {
            value = rs.getClob(columName);
        } else {
            // Some unknown type desired -> rely on getObject.
            value = getResultSetValue(rs, columName);
        }

        // Perform was-null check if demanded (for results that the
        // JDBC driver returns as primitives).
        if (wasNullCheck && value != null && rs.wasNull()) {
            value = null;
        }
        return (T) value;
    }

    /**
     * @param columnName
     *            the SQL name of the column
     * @see #getResultSetValue(ResultSet, int)
     */
    public static Object getResultSetValue(ResultSet rs, String columName) throws SQLException {
        Object obj = rs.getObject(columName);
        String className = null;
        if (obj != null) {
            className = obj.getClass().getName();
        }
        if (obj instanceof Blob) {
            obj = rs.getBytes(columName);
        } else if (obj instanceof Clob) {
            obj = rs.getString(columName);
        } else if (className != null
                && ("oracle.sql.TIMESTAMP".equals(className) || "oracle.sql.TIMESTAMPTZ".equals(className))) {
            obj = rs.getTimestamp(columName);
        } else if (className != null && className.startsWith("oracle.sql.DATE")) {
            int columnIndex = lookupColumnIndex(rs.getMetaData(), columName);
            String metaDataClassName = rs.getMetaData().getColumnClassName(columnIndex);
            if ("java.sql.Timestamp".equals(metaDataClassName) || "oracle.sql.TIMESTAMP".equals(metaDataClassName)) {
                obj = rs.getTimestamp(columName);
            } else {
                obj = rs.getDate(columName);
            }
        } else if (obj != null && obj instanceof java.sql.Date) {
            int columnIndex = lookupColumnIndex(rs.getMetaData(), columName);
            if ("java.sql.Timestamp".equals(rs.getMetaData().getColumnClassName(columnIndex))) {
                obj = rs.getTimestamp(columName);
            }
        }
        return obj;
    }

    /**
     * @return the index of the matching column name
     * @param resultSetMetaData
     * @param columnName
     *            the SQL name of the column
     */
    public static int lookupColumnIndex(ResultSetMetaData resultSetMetaData, String name) throws SQLException {
        int columnCount = resultSetMetaData.getColumnCount();
        for (int i = 1; i <= columnCount; i++) {
            String columnName = resultSetMetaData.getColumnName(i);
            if (columnName.equals(name)) {
                return i;
            }
        }
        return -1;
    }
    
}
